import { expandGlobBase } from './inject';
import { getResourcesFromList } from './index';

describe('Resouce handling - ', () => {
  it(`loading`, () => {
    // dummy
  })
  // it(`loading`, () => {
  //   const resources = getResourcesFromList(require(`../package.json`), 'aurelia.build.resources')
  //   // console.log(resources)
  //   expect(resources).toBeTruthy()
  //   expect(resources.length).toBe(6)
  // })
  // it(`globbing`, (done) => {
  //   expandGlobBase
  //   const resources = getResourcesFromList(require(`../package.json`), '_test.resources')
  //   // console.log(resources)
  //   expect(resources).toBeTruthy()
  //   expect(resources.length).toBe(4)
  // })
})
