declare module 'webpack/lib/dependencies/ModuleDependency' {
  export = Webpack.Core.ModuleDependency
}

declare module 'webpack/lib/dependencies/Module' {
  export = Webpack.Core.Module
}

declare module 'webpack/lib/NormalModule' {
  export = Webpack.Core.NormalModule
}

declare module 'webpack-sources/lib/ReplaceSource' {
  export = Webpack.WebpackSources.ReplaceSource
}

declare module 'webpack-sources/lib/Source' {
  export = Webpack.WebpackSources.Source
}
