/// <reference types="source-map" />
import * as SourceMap from 'source-map';
export default function ListBasedRequireLoader(this: Webpack.Core.LoaderContext, source: string, sourceMap?: SourceMap.RawSourceMap): Promise<void>;
