"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
describe('Resouce handling - ', function () {
    it("loading", function () {
        // dummy
    });
    // it(`loading`, () => {
    //   const resources = getResourcesFromList(require(`../package.json`), 'aurelia.build.resources')
    //   // console.log(resources)
    //   expect(resources).toBeTruthy()
    //   expect(resources.length).toBe(6)
    // })
    // it(`globbing`, (done) => {
    //   expandGlobBase
    //   const resources = getResourcesFromList(require(`../package.json`), '_test.resources')
    //   // console.log(resources)
    //   expect(resources).toBeTruthy()
    //   expect(resources.length).toBe(4)
    // })
});
//# sourceMappingURL=index.spec.js.map