"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var loaderUtils = require("loader-utils");
var inject_1 = require("../utils/inject");
var utils_1 = require("../utils");
var htmlLoader = require("html-loader");
var debug = require("debug");
var log = debug('html-require-loader');
exports.htmlRequireDefaults = {
    selectorsAndAttributes: [
        // e.g. <require from="./file">
        // e.g. <require from="bootstrap" lazy bundle="vendor">
        { selector: 'require', attribute: 'from' },
        // e.g. <compose view-model="file">
        { selector: '[view-model]', attribute: 'view-model' },
        // e.g. <compose view="file">
        { selector: '[view]', attribute: 'view' },
    ],
    // by default glob template string: e.g. '${anything}'
    globReplaceRegex: /\${.+?}/g,
    enableGlobbing: true
};
function HtmlRequireLoader(pureHtml, sourceMap) {
    var _this = this;
    if (this.cacheable) {
        this.cacheable();
    }
    var query = Object.assign({}, exports.htmlRequireDefaults, loaderUtils.parseQuery(this.query));
    var source = htmlLoader.bind(this)(pureHtml, sourceMap);
    try {
        var resources_1 = utils_1.getTemplateResourcesData(pureHtml, query.selectorsAndAttributes, query.globReplaceRegex);
        if (!resources_1.length) {
            return source;
        }
        return (function () { return __awaiter(_this, void 0, void 0, function () {
            var resourceData, requireStrings, inject;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.async();
                        return [4 /*yield*/, inject_1.addBundleLoader(resources_1)];
                    case 1:
                        resourceData = _a.sent();
                        log("Adding resources to " + this.resourcePath + ": " + resourceData.map(function (r) { return r.literal; }).join(', '));
                        if (!query.enableGlobbing) return [3 /*break*/, 3];
                        return [4 /*yield*/, inject_1.expandAllRequiresForGlob(resourceData, this)];
                    case 2:
                        resourceData = _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        resourceData = resourceData.filter(function (r) { return !r.literal.includes("*"); });
                        _a.label = 4;
                    case 4: return [4 /*yield*/, inject_1.getRequireStrings(resourceData, query.addLoadersCallback, this)];
                    case 5:
                        requireStrings = _a.sent();
                        inject = requireStrings.map(inject_1.wrapInRequireInclude).join('\n');
                        return [2 /*return*/, inject_1.appendCodeAndCallback(this, source, inject, sourceMap)];
                }
            });
        }); })().catch(function (e) {
            log(e);
            _this.emitError(e.message);
            return _this.callback(undefined, source, sourceMap);
        });
    }
    catch (e) {
        log(e);
        this.emitError(e.message);
        return source;
    }
}
exports.default = HtmlRequireLoader;
//# sourceMappingURL=html-require-loader.js.map